/*
 * Copyright (c) 2008, intarsys consulting GmbH
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * - Neither the name of intarsys nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package de.intarsys.cwt.image;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import javax.imageio.IIOException;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataFormatImpl;
import javax.imageio.stream.ImageInputStream;

import org.w3c.dom.Node;

public class ImageTools {

	public static String extractJPEGColorSpaceType(ImageReader reader)
			throws IOException {
		IIOMetadata metadata;
		Node node;
		Node allegedChromaNode;
		Node allegedColorNode;
		Node name;

		try {
			metadata = reader.getImageMetadata(0);
		} catch (RuntimeException ex) {
			throw new ImageException(ex);
		} catch (IIOException ex) {
			throw new ImageException(ex);
		}
		node = metadata
				.getAsTree(IIOMetadataFormatImpl.standardMetadataFormatName);
		/*
		 * expect wanted nodes to always be there and to always have the needed
		 * attribute
		 */
		allegedChromaNode = node.getFirstChild();
		while (!allegedChromaNode.getLocalName().equals("Chroma")) { //$NON-NLS-1$
			allegedChromaNode = allegedChromaNode.getNextSibling();
		}
		allegedColorNode = allegedChromaNode.getFirstChild();
		while (!allegedColorNode.getLocalName().equals("ColorSpaceType")) { //$NON-NLS-1$
			allegedColorNode = allegedColorNode.getNextSibling();
		}
		if ((name = allegedColorNode.getAttributes().getNamedItem("name")) == null) { //$NON-NLS-1$
			throw new ImageException("color space not declared"); //$NON-NLS-1$
		}
		return name.getNodeValue();
	}

	public static String extractJPEGColorSpaceType(InputStream stream)
			throws IOException {
		ImageInputStream imageStream;

		imageStream = ImageIO.createImageInputStream(stream);

		try {
			Iterator<ImageReader> readers;
			ImageReader reader;

			readers = ImageIO.getImageReaders(imageStream);
			if (!readers.hasNext()) {
				throw new ImageException("no reader for this image type");
			}
			reader = readers.next();
			try {
				imageStream.reset();
				reader.setInput(imageStream);
				return extractJPEGColorSpaceType(reader);
			} finally {
				reader.dispose();
			}
		} finally {
			imageStream.close();
		}
	}

}
