/*
 * Copyright (c) 2008, intarsys consulting GmbH
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * - Neither the name of intarsys nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package de.intarsys.cwt.font.truetype;

/**
 * 
 */
public class TTPostScriptInformation {
	private float version = 0; // 0x00010000 for version 1.0

	private float italicAngle; // Italic angle in counter-clockwise degrees

	// from the vertical. Zero for upright text,
	// negative for text that leans to the right
	// (forward).

	private int underlinePosition; // This is the suggested distance of the top

	// of the underline from the baseline
	// (negative values indicate below
	// baseline).

	private int underlineThickness; // Suggested values for the underline

	// thickness.

	private long isFixedPitch; // Set to 0 if the font is proportionally

	// spaced, non-zero if the font is not
	// proportionally spaced (i.e. monospaced).

	public TTPostScriptInformation() {
	}

	public long getIsFixedPitch() {
		return isFixedPitch;
	}

	public float getItalicAngle() {
		return italicAngle;
	}

	public int getUnderlinePosition() {
		return underlinePosition;
	}

	public int getUnderlineThickness() {
		return underlineThickness;
	}

	public float getVersion() {
		return version;
	}

	public void setIsFixedPitch(long l) {
		isFixedPitch = l;
	}

	public void setItalicAngle(float f) {
		italicAngle = f;
	}

	public void setUnderlinePosition(int i) {
		underlinePosition = i;
	}

	public void setUnderlineThickness(int i) {
		underlineThickness = i;
	}

	public void setVersion(float f) {
		version = f;
	}
}
